@@ -5,7 +5,9 @@
 import random
 from itertools import product
 import json
+
 from joblib import Parallel, delayed
+from tqdm import tqdm
 import numpy as np
 import cv2
 from pathlib import Path
@@ -18,8 +20,9 @@ class Patcher:
 
     Args:
         slide (wsiprocess.slide.Slide): Slide object.
-        method (str): Method name to run. One of {"none", "classification",
-            "detection", "segmentation}. Characters are converted to lowercase.
+        method (str): Method name to run. One of {"evaluation",
+            "classification", "detection", "segmentation}. Characters are
+            converted to lowercase.
         annotation (wsiprocess.annotation.Annotation, optional):
             Annotation object.
         save_to (str, optional): The root of the output directory.
@@ -43,6 +46,7 @@ class Patcher:
             "no_patches" is set, Patcher extracts patches.
         no_patches (bool, optional): If set, Patcher runs without extracting
             patches and saves them to disk.
+        verbose (bool, optional): If set, a progress bar appears when patching.
 
     Attributes:
         slide (wsiprocess.slide.Slide): Slide object.
@@ -50,8 +54,8 @@ class Patcher:
         wsi_height (int): Height of the slide.
         filepath (str): Path to the whole slide image.
         filestem (str): Stem of the file name.
-        method (str): Method name to run. One of {"none", "classification",
-            "detection", "segmentation}
+        method (str): Method name to run. One of {"evaluation",
+            "classification", "detection", "segmentation}
         annotation (wsiprocess.annotation.Annotation):
             Annotation object.
         masks (dict): Masks to show the location of classes.
@@ -73,6 +77,7 @@ class Patcher:
             finish.
         extract_patches (bool): Whether to save patches when Patcher runs.
         no_patches (bool): Whether to save patches when Patcher runs.
+        verbose (bool, optional): If set, a progress bar appears when patching.
 
         x_lefttop (list): Offsets of patches to the x-axis direction except for
             the right edge.
@@ -90,12 +95,12 @@ class Patcher:
             patch_width=256, patch_height=256, overlap_width=0,
             overlap_height=0, offset_x=0, offset_y=0, on_foreground=0.5,
             on_annotation=0.5, start_sample=False, finished_sample=False,
-            no_patches=False, crop_bbox=False):
+            no_patches=False, crop_bbox=False, verbose=False):
         self.verify = Verify(
             save_to, slide.filestem, method, start_sample, finished_sample,
             no_patches, crop_bbox)
         self.verify.sizes(
-            slide.wsi_width, slide.wsi_height, offset_x, offset_y,
+            slide.width, slide.height, offset_x, offset_y,
             patch_width, patch_height, overlap_width, overlap_height,
             annotation.dot_bbox_width, annotation.dot_bbox_height)
         self.verify.on_params(on_annotation, on_foreground)
@@ -105,8 +110,8 @@ class Patcher:
         self.filepath = slide.filename
         self.filestem = slide.filestem
         self.method = method.lower()
-        self.wsi_width = slide.wsi_width
-        self.wsi_height = slide.wsi_height
+        self.wsi_width = slide.width
+        self.wsi_height = slide.height
         self.p_width = int(patch_width)
         self.p_height = int(patch_height)
         self.p_area = patch_width * patch_height
@@ -124,14 +129,18 @@ class Patcher:
             self.offset_y,
             self.wsi_height,
             patch_height - overlap_height)][:-1]
-        self.iterator = list(product(self.x_lefttop, self.y_lefttop))
+
         self.last_x = self.slide.width - patch_width
         self.last_y = self.slide.height - patch_height
 
+        self.get_iterator()
+
         self.start_sample = start_sample
         self.finished_sample = finished_sample
         self.no_patches = no_patches
 
+        self.verbose = verbose
+
         self.on_foreground = on_foreground
         self.annotation = annotation
         if annotation:
@@ -148,7 +157,16 @@ class Patcher:
         self.result = {"result": []}
 
     def __str__(self):
-        return "wsiprocess.patcher.Patcher {}".format(self.slide.filename)
+        return "wsiprocess.patcher.Patcher {}".format(self.slide.path)
+
+    def get_iterator(self):
+        self.iterator = list(product(self.x_lefttop, self.y_lefttop))
+        bottomedge = [(x, self.last_y) for x in self.x_lefttop]
+        rightedge = [(self.last_x, y) for y in self.y_lefttop]
+        rightbottom = [(self.last_x, self.last_y)]
+        self.iterator += bottomedge
+        self.iterator += rightedge
+        self.iterator += rightbottom
 
     def save_patch_result(self, x, y, cls):
         """Save the extracted patch data to result
@@ -159,7 +177,7 @@ class Patcher:
             cls (str): Class of the patch or the bounding box or the segmented
                 area.
         """
-        if self.method == "none":
+        if self.method == "evaluation":
             self.result["result"].append({"x": x,
                                           "y": y,
                                           "w": self.p_width,
@@ -280,7 +298,8 @@ class Patcher:
         left_on_patch = (patch_left <= bblefts) & (bblefts <= patch_right)
         right_on_patch = (patch_left <= bbrights) & (bbrights <= patch_right)
         top_on_patch = (patch_top <= bbtops) & (bbtops <= patch_bottom)
-        bottom_on_patch = (patch_top <= bbbottoms) & (bbbottoms <= patch_bottom)
+        bottom_on_patch = (patch_top <= bbbottoms) & (
+            bbbottoms <= patch_bottom)
 
         topleft_on_patch = left_on_patch & top_on_patch
         topright_on_patch = right_on_patch & top_on_patch
@@ -317,7 +336,8 @@ class Patcher:
         left_on_patch = (patch_left <= bblefts) & (bblefts <= patch_right)
         right_on_patch = (patch_left <= bbrights) & (bbrights <= patch_right)
         top_on_patch = (patch_top <= bbtops) & (bbtops <= patch_bottom)
-        bottom_on_patch = (patch_top <= bbbottoms) & (bbbottoms <= patch_bottom)
+        bottom_on_patch = (patch_top <= bbbottoms) & (
+            bbbottoms <= patch_bottom)
 
         # One side is on the patch
         leftside_on_patch = (left_on_patch) &\
@@ -497,10 +517,10 @@ class Patcher:
             on_annotation_classes = ["foreground"]
         for cls in on_annotation_classes:
             if not self.no_patches:
-                patch = self.slide.slide.crop(
+                patch = self.slide.crop(
                     x, y, self.p_width, self.p_height)
-                patch.jpegsave(
-                    "{}/{}/patches/{}/{:06}_{:06}.jpg".format(
+                patch.save(
+                    "{}/{}/patches/{}/{:06}_{:06}.png".format(
                         self.save_to, self.filestem, cls, x, y))
             self.save_patch_result(x, y, cls)
 
@@ -523,18 +543,25 @@ class Patcher:
         if self.start_sample:
             self.get_random_sample("start", 3)
 
-        parallel = Parallel(n_jobs=cores, backend="threading", verbose=0)
+        def Runner(**joblib_args):
+            def run(**tqdm_args):
+                def tmp(op_iter):
+                    def progress(args): return lambda x: tqdm(x, **args)
+                    bar = progress(tqdm_args)
+                    return Parallel(**joblib_args)(bar(op_iter))
+                return tmp
+            return run
+
+        if self.verbose:
+            run = Runner(n_jobs=cores, backend="threading", verbose=0)
+            runner = run(
+                desc=f"[{self.slide.filename} {self.p_width}x{self.p_height}]")
+        else:
+            runner = Parallel(n_jobs=cores, backend="threading", verbose=0)
+
         # from the left top to just before the right bottom.
-        parallel([delayed(self.get_patch)(x, y, classes)
-                  for x, y in self.iterator])
-        # the bottom edge.
-        parallel([delayed(self.get_patch)(x, self.last_y, classes)
-                  for x in self.x_lefttop])
-        # the right edge
-        parallel([delayed(self.get_patch)(self.last_x, y, classes)
-                  for y in self.y_lefttop])
-        # right bottom patch
-        self.get_patch(self.last_x, self.last_y, classes)
+        runner([delayed(self.get_patch)(x, y, classes)
+                for x, y in self.iterator])
 
         # save results
         self.save_results()
@@ -551,13 +578,13 @@ class Patcher:
         for patch in self.result["result"]:
             for bb in patch["bbs"]:
                 if bb["class"] in classes:
-                    mini_patch = self.slide.slide.crop(
+                    mini_patch = self.slide.crop(
                         patch["x"] + bb["x"],
                         patch["y"] + bb["y"],
                         bb["w"],
                         bb["h"])
-                    mini_patch.jpegsave(
-                        "{}/{}/mini_patches/{}/{:06}_{:06}.jpg".format(
+                    mini_patch.save(
+                        "{}/{}/mini_patches/{}/{:06}_{:06}.png".format(
                             self.save_to, self.filestem, bb["class"],
                             patch["x"] + bb["x"], patch["y"] + bb["y"]))
 
@@ -600,7 +627,7 @@ class Patcher:
         for i in range(sample_count):
             x = random.choice(self.x_lefttop)
             y = random.choice(self.y_lefttop)
-            patch = self.slide.slide.crop(x, y, self.p_width, self.p_height)
-            patch.pngsave(
+            patch = self.slide.crop(x, y, self.p_width, self.p_height)
+            patch.save(
                 "{}/{}/{}_sample/{:06}_{:06}.png".format(
                     self.save_to, self.filestem, phase, x, y))
